@@ -253,7 +253,7 @@ void anonymize_posts::operator()(post_t& post)
     xact.copy_details(*post.xact);
 
     std::ostringstream buf;
-    buf << reinterpret_cast<uintmax_t>(post.xact->payee.c_str())
+    buf << reinterpret_cast<boost::uintmax_t>(post.xact->payee.c_str())
         << integer_gen() << post.xact->payee.c_str();
 
 		sha.reset();
@@ -894,16 +894,7 @@ void subtotal_posts::operator()(post_t& post)
   account_t * acct = post.reported_account();
   assert(acct);
 
-#if 0
-  // jww (2012-04-06): The problem with doing this early is that
-  // fn_display_amount will recalculate this again.  For example, if you
-  // use --invert, it will invert both here and in the display amount,
-  // effectively negating it.
-  bind_scope_t bound_scope(*amount_expr.get_context(), post);
-  value_t amount(amount_expr.calc(bound_scope));
-#else
   value_t amount(post.amount);
-#endif
 
   post.xdata().compound_value = amount;
   post.xdata().add_flags(POST_EXT_COMPOUND);
@@ -1100,17 +1091,17 @@ void posts_as_equity::report_subtotal()
     value_t value(pair.second.value.strip_annotations(report.what_to_keep()));
     if (! value.is_zero()) {
       if (value.is_balance()) {
-        foreach (const balance_t::amounts_map::value_type& amount_pair,
-                 value.as_balance_lval().amounts) {
-          if (! amount_pair.second.is_zero())
-            handle_value(/* value=      */ amount_pair.second,
-                         /* account=    */ pair.second.account,
-                         /* xact=       */ &xact,
-                         /* temps=      */ temps,
-                         /* handler=    */ handler,
-                         /* date=       */ finish,
-                         /* act_date_p= */ false);
-        }
+        value.as_balance_lval().map_sorted_amounts
+          ([&](const amount_t& amt) {
+             if (! amt.is_zero())
+               handle_value(/* value=      */ amt,
+                            /* account=    */ pair.second.account,
+                            /* xact=       */ &xact,
+                            /* temps=      */ temps,
+                            /* handler=    */ handler,
+                            /* date=       */ finish,
+                            /* act_date_p= */ false);
+           });
       } else {
         handle_value(/* value=      */ value.to_amount(),
                      /* account=    */ pair.second.account,
