@@ -48,7 +48,6 @@
 #include "convert.h"
 #include "ptree.h"
 #include "emacs.h"
-#include "org.h"
 
 namespace ledger {
 
@@ -564,6 +563,14 @@ value_t report_t::fn_should_bold(call_scope_t& scope)
     return false;
 }
 
+value_t report_t::fn_averaged_lots(call_scope_t& args)
+{
+  if (args.has<balance_t>(0))
+    return average_lot_prices(args.get<balance_t>(0));
+  else
+    return args[0];
+}
+
 value_t report_t::fn_market(call_scope_t& args)
 {
   value_t result;
@@ -841,6 +848,25 @@ value_t report_t::fn_commodity(call_scope_t& args)
   return string_value(args.get<amount_t>(0).commodity().symbol());
 }
 
+value_t report_t::fn_commodity_price(call_scope_t& args)
+{
+  optional<price_point_t> price_point
+    = commodity_pool_t::current_pool->commodity_price_history.find_price
+        (args.get<amount_t>(0).commodity(), args.get<datetime_t>(1));
+  if (price_point) {
+    return price_point->price;
+  } else {
+    return amount_t();
+  }
+}
+
+value_t report_t::fn_set_commodity_price(call_scope_t& args)
+{
+  args.get<amount_t>(0).commodity().add_price(
+    args.get<datetime_t>(1), args.get<amount_t>(2), true);
+  return NULL_VALUE;
+}
+
 value_t report_t::fn_clear_commodity(call_scope_t& args)
 {
   amount_t amt(args.get<amount_t>(0));
@@ -1131,6 +1157,7 @@ option_t<report_t> * report_t::lookup_option(const char * p)
     else OPT(average);
     else OPT(account_width_);
     else OPT(amount_width_);
+    else OPT(average_lot_prices);
     break;
   case 'b':
     OPT(balance_format_);
@@ -1371,6 +1398,8 @@ expr_t::ptr_op_t report_t::lookup(const symbol_t::kind_t kind,
         return MAKE_FUNCTOR(report_t::fn_ansify_if);
       else if (is_eq(p, "abs"))
         return MAKE_FUNCTOR(report_t::fn_abs);
+      else if (is_eq(p, "averaged_lots"))
+        return MAKE_FUNCTOR(report_t::fn_averaged_lots);
       break;
 
     case 'b':
@@ -1389,6 +1418,8 @@ expr_t::ptr_op_t report_t::lookup(const symbol_t::kind_t kind,
         return WRAP_FUNCTOR(fn_cyan);
       else if (is_eq(p, "commodity"))
         return MAKE_FUNCTOR(report_t::fn_commodity);
+      else if (is_eq(p, "commodity_price"))
+        return MAKE_FUNCTOR(report_t::fn_commodity_price);
       else if (is_eq(p, "ceiling"))
         return MAKE_FUNCTOR(report_t::fn_ceiling);
       else if (is_eq(p, "clear_commodity"))
@@ -1491,6 +1522,8 @@ expr_t::ptr_op_t report_t::lookup(const symbol_t::kind_t kind,
         return MAKE_FUNCTOR(report_t::fn_strip);
       else if (is_eq(p, "should_bold"))
         return MAKE_FUNCTOR(report_t::fn_should_bold);
+      else if (is_eq(p, "set_commodity_price"))
+        return MAKE_FUNCTOR(report_t::fn_set_commodity_price);
       break;
 
     case 't':
@@ -1672,11 +1705,6 @@ expr_t::ptr_op_t report_t::lookup(const symbol_t::kind_t kind,
       if (is_eq(p, "lisp"))
         return POSTS_REPORTER(new format_emacs_posts(output_stream));
       break;
-    case 'o':
-      if (is_eq(p, "org"))
-        return POSTS_REPORTER(new posts_to_org_table
-                            (*this, maybe_format(HANDLER(prepend_format_))));
-      break;
 
     case 'p':
       if (*(p + 1) == '\0' || is_eq(p, "print")) {
