@@ -319,7 +319,9 @@ std::streamsize instance_t::read_line(char *& line)
     context.curr_pos  = context.line_beg_pos;
     context.curr_pos += len;
 
-    if (context.linenum == 0 && utf8::is_bom(context.linebuf)) {
+    if (context.linenum == 0 &&
+        utf8::starts_with_bom(
+          context.linebuf, context.linebuf + sizeof(context.linebuf))) {
       line = &context.linebuf[3];
       len -= 3;
     } else {
@@ -808,6 +810,8 @@ void instance_t::apply_directive(char * line)
 {
   char * b = next_element(line);
   string keyword(line);
+  if (! b)
+    throw_(parse_error, _f("Directive 'apply %1%' requires an argument") % keyword);
   if (keyword == "account")
     apply_account_directive(b);
   else if (keyword == "tag")
@@ -1036,7 +1040,7 @@ void instance_t::payee_directive(char * line)
 
     if (keyword == "alias")
       payee_alias_directive(payee, b);
-    if (keyword == "uuid")
+    else if (keyword == "uuid")
       payee_uuid_directive(payee, b);
   }
 }
@@ -1440,7 +1444,7 @@ post_t * instance_t::parse_post(char *          line,
 
   // Parse the account name
 
-  if (! *p)
+  if (! *p || *p == ';')
     throw parse_error(_("Posting has no account"));
 
   char * next = next_element(p, true);
@@ -1631,7 +1635,7 @@ post_t * instance_t::parse_post(char *          line,
 
       const amount_t& amt(*post->assigned_amount);
       value_t account_total
-        (post->account->amount().strip_annotations(keep_details_t()));
+        (post->account->amount(!post->has_flags(POST_VIRTUAL)).strip_annotations(keep_details_t()));
 
       DEBUG("post.assign", "line " << context.linenum << ": "
             << "account balance = " << account_total);
@@ -1664,7 +1668,7 @@ post_t * instance_t::parse_post(char *          line,
 
       // Subtract amounts from previous posts to this account in the xact.
       for (post_t* p : xact->posts) {
-        if (p->account == post->account) {
+        if (p->account == post->account && p->has_flags(POST_VIRTUAL) == post->has_flags(POST_VIRTUAL)) {
           diff -= p->amount;
           DEBUG("textual.parse", "line " << context.linenum << ": "
                 << "Subtracting " << p->amount << ", diff = " << diff);
